@@ -40,6 +40,7 @@ namespace Beagle.Tile {
 		static private long uidSrc = 0;
 
 		private long uid;
+		private Uri uri;
 		private Query query;
 		
 		public Tile ()
@@ -54,6 +55,11 @@ namespace Beagle.Tile {
 			get { return "_tile_" + uid; }
 		}
 
+		public Uri Uri {
+			get { return uri; }
+			set { uri = value; }
+		}
+
 		public Query Query {
 			get { return query; }
 			set { query = value; }
