@@ -147,9 +147,16 @@ namespace Beagle.Daemon {
 		private static void LogMemoryUsage ()
 		{
 			while (! Shutdown.ShutdownRequested) {
+				int vm_size = SystemInformation.VmSize;
+
 				Logger.Log.Debug ("Memory usage: VmSize={0}  GC.GetTotalMemory={1}",
-						  SystemInformation.VmSize,
-						  GC.GetTotalMemory (false));
+						  vm_size, GC.GetTotalMemory (false));
+
+				if (vm_size > 200 * 1024) {
+					Logger.Log.Debug ("VmSize too large --- shutting down");
+					Shutdown.BeginShutdown ();
+				}
+
 				Thread.Sleep (1000);
 			}
 		}
@@ -164,6 +171,7 @@ namespace Beagle.Daemon {
 			bool arg_debug_memory = false;
 			bool arg_network = false;
 			bool arg_fg = false;
+			bool arg_disable_scheduler = false;
 			int arg_port = 0;
 
 			int i = 0;
@@ -219,6 +227,10 @@ namespace Beagle.Daemon {
 					arg_network = true;
 					break;
 
+				case "--disable-scheduler":
+					arg_disable_scheduler = true;
+					break;
+
 				case "--port":
 					try {
 						arg_port = Convert.ToInt32(next_arg);
@@ -344,8 +356,10 @@ namespace Beagle.Daemon {
 			QueryDriver.Start ();
 
 			// Start the Global Scheduler thread
-			Logger.Log.Debug ("Starting Scheduler thread");
-			Scheduler.Global.Start ();
+			if (! arg_disable_scheduler) {
+				Logger.Log.Debug ("Starting Scheduler thread");
+				Scheduler.Global.Start ();
+			}
 
 			// Start our Inotify threads
 			Logger.Log.Debug ("Starting Inotify threads");
