@@ -43,6 +43,7 @@ class QueryTool {
 	// CLI args
 	static bool keepRunning = false;
 	static bool verbose = false;
+	static bool display_hits = true;
 
 	static void OnHitsAdded (Query source, ICollection hits)
 	{
@@ -53,6 +54,11 @@ class QueryTool {
 					   (lastQueryTime - queryStartTime).TotalSeconds);
 		}
 
+		if (! display_hits) {
+			count += hits.Count;
+			return;
+		}
+
 		foreach (Hit hit in hits) {
 			if (verbose)
 				Console.WriteLine ("  Uri: {0}", hit.Uri);
@@ -60,6 +66,8 @@ class QueryTool {
 				Console.WriteLine (hit.Uri);
 
 			if (verbose) {
+				string snippet = query.GetSnippet (hit.Uri);
+				Console.WriteLine (" Snip: {0}", snippet != null ? snippet : "(null)");
 				Console.WriteLine (" Type: {0}", hit.Type);
 				Console.WriteLine ("MimeT: {0}", hit.MimeType == null ? "(null)" : hit.MimeType);
 				Console.WriteLine ("  Src: {0}", hit.Source);
@@ -83,6 +91,9 @@ class QueryTool {
 	{
 		lastQueryTime = DateTime.Now;
 
+		if (! display_hits)
+			return;
+
 		foreach (Uri uri in uris) {
 			Console.WriteLine ("Subtracted Uri '{0}'", uri);
 			Console.WriteLine ();
@@ -93,9 +104,12 @@ class QueryTool {
 
 	static void OnFinished (QueryProxy query)
 	{
-		if (verbose)
+		if (verbose) {
 			Console.WriteLine ("Elapsed time: {0:0.000}s",
 					   (DateTime.Now - queryStartTime).TotalSeconds);
+			Console.WriteLine ("Total hits: {0}", count);
+		}
+
 		Gtk.Application.Quit ();
 	}
 
@@ -117,6 +131,8 @@ class QueryTool {
 			"                   \t\tSources list available from beagle-status.\n" +
 			"  --live-query\t\t\tRun continuously, printing notifications if a\n" +
 			"              \t\t\tquery changes.\n" +
+			"  --stats-only\t\t\tOnly display statistics about the query, not\n" +
+			"              \t\t\tthe actual results.\n" +
 			"  --help\t\t\tPrint this usage message.\n";
 
 		Console.WriteLine (usage);
@@ -167,6 +183,10 @@ class QueryTool {
 			case "--verbose":
 				verbose = true;
 				break;
+			case "--stats-only":
+				verbose = true;
+				display_hits = false;
+				break;
 
 			case "--help":
 			case "--usage":
@@ -184,9 +204,8 @@ class QueryTool {
 		if (! keepRunning)
 			query.FinishedEvent += OnFinished;
 
-		query.Start ();
-
 		queryStartTime = DateTime.Now;
+		query.Start ();
 
 		Gtk.Application.Run ();
 	}
